/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.db.test;

import java.util.Set;
import org.openide.modules.Dependency;
import org.openide.modules.SpecificationVersion;
import org.openide.util.ContextGlobalProvider;
import org.openide.util.Lookup;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.junit.BeforeClass;
import java.awt.Dialog;
import javax.swing.JDialog;
import org.netbeans.junit.MockServices;
import org.openide.DialogDescriptor;
import org.openide.DialogDisplayer;
import org.openide.NotifyDescriptor;
import org.openide.modules.ModuleInfo;
import org.openide.util.lookup.ProxyLookup;
import static org.mockito.Mockito.*;

/**
 *
 * @author Timon Veenstra
 */
public abstract class BaseContextTest {

    protected static final String TEST_MODULE = "test";

    @BeforeClass
    public static void setUpClass() throws Exception {
        for (Object o : MockContextGlobalProvider.LOOKUP.lookupAll(Object.class)){
            MockContextGlobalProvider.remove(o);
        }
        MockServices.setServices(MockDialogDisplayer.class, MockContextGlobalProvider.class,TestModuleInfo.class);
     
    }
    
    /**
     * A dummy module info to fool the guiEventBus that the module TEST_MODULE is present in the application
     * so the listener returning that module name will receive the message
     * 
     */
    public static final class TestModuleInfo extends ModuleInfo{

        @Override
        public String getDisplayName() {
            return TEST_MODULE;
        }

        
        @Override
        public String getCodeNameBase() {
            return null;
        }

        @Override
        public int getCodeNameRelease() {
            return 0;
        }

        @Override
        public String getCodeName() {
            return null;
        }

        @Override
        public SpecificationVersion getSpecificationVersion() {
            return null;
        }

        @Override
        public boolean isEnabled() {
            return true;
        }

        @Override
        public Object getAttribute(String attr) {
            return null;
        }

        @Override
        public Object getLocalizedAttribute(String attr) {
            return null;
        }

        @Override
        public Set<Dependency> getDependencies() {
            return null;
        }

        @Override
        public boolean owns(Class<?> clazz) {
            return true;
        }
        
    }

    public static final class MockDialogDisplayer extends DialogDisplayer {
 
        public static Object[] options;

        @Override
        public Object notify(NotifyDescriptor descriptor) {
            throw new UnsupportedOperationException("notify Not supported yet.");
        }

        @Override
        public Dialog createDialog(DialogDescriptor descriptor) {
            options = descriptor.getOptions();
            return mock(JDialog.class);
        }
    }
     
   /**	
    * A mock for ContextglobalProvider.	
    * Attention!!!: When a object added to this context can't be found with Utilities.actionsGlobalContext().lookup(someClass) there is probably an	
    * offending test which has already executed Utilities.actionsGlobalContext(). This can't be reset. to fix this problem just find the offending class	
    * Mostly a service class. Let it extend BaseContextTest and your problem is fixed	
    */
    public static final class MockContextGlobalProvider implements ContextGlobalProvider {

        private static final InstanceContent CONTENT = new InstanceContent();
        static final Lookup LOOKUP = new AbstractLookup(CONTENT);

        public MockContextGlobalProvider() {
        }
        
        
        @Override
        public Lookup createGlobalContext() {
            return LOOKUP;
        }

        public static void add(Object o) {
            CONTENT.add(o);
        }

        public static void remove(Object o) {
            CONTENT.remove(o);
        }
        public static Lookup getLookup() {
   
            return LOOKUP;
        }
    }
}
